@@ -1,9 +1,13 @@
+# with input in the DataFrame format
+# handle categorical values
+
 import rpi_feature_selection_toolbox
 import os, sys
 # from typing import NamedTuple, Union, List, Sequence, Any, Dict
 import typing
 import scipy.io
 import numpy as np
+from sklearn import preprocessing
 
 from d3m import container
 from d3m.metadata import base as metadata_base
@@ -15,8 +19,11 @@ from d3m.primitive_interfaces.base import CallResult
 # from . import __version__
 
 
-Inputs = container.ndarray
-Outputs = container.ndarray
+#Inputs = container.ndarray
+#Outputs = container.ndarray
+
+Inputs = container.DataFrame
+Outputs = container.DataFrame
 
 __all__ = ('IPCMBplus_Selector',)
 
@@ -57,7 +64,7 @@ class IPCMBplus_Selector(SupervisedLearnerPrimitiveBase[Inputs, Outputs, Params,
             {
                 'type': metadata_base.PrimitiveInstallationType.PIP,
                 'package': 'rpi_featureSelection_python_tools',
-                'version': '2.2.1'
+                'version': '2.2.2'
             }
         ],
         'python_path': 'd3m.primitives.rpi_featureSelection_python_tools.IPCMBplus_Selector',
@@ -65,9 +72,6 @@ class IPCMBplus_Selector(SupervisedLearnerPrimitiveBase[Inputs, Outputs, Params,
             metadata_base.PrimitiveAlgorithmType.MINIMUM_REDUNDANCY_FEATURE_SELECTION
         ],
         'primitive_family': metadata_base.PrimitiveFamily.FEATURE_SELECTION,
-        'preconditions': [
-            "NO_CATEGORICAL_VALUES"
-        ]
     })
 
 
@@ -79,8 +83,32 @@ class IPCMBplus_Selector(SupervisedLearnerPrimitiveBase[Inputs, Outputs, Params,
         self._fitted = False
 
     def set_training_data(self, *, inputs: Inputs, outputs: Outputs) -> None:
-        self._training_inputs = inputs
-        self._training_outputs = outputs
+        # convert categorical values to numerical values for training labels
+        LE = preprocessing.LabelEncoder()
+        LE = LE.fit(outputs.iloc[:,0])
+        self._training_outputs = LE.transform(outputs.iloc[:,0])
+        
+        # convert cateforical values to numerical values in training data
+        metadata = inputs.metadata
+        [m,n] = inputs.shape
+        self._training_inputs = np.zeros((m,n))
+        for column_index in metadata.get_elements((metadata_base.ALL_ELEMENTS,)):
+            if column_index is metadata_base.ALL_ELEMENTS: 
+                continue
+            column_metadata = metadata.query((metadata_base.ALL_ELEMENTS, column_index))
+            semantic_types = column_metadata.get('semantic_types', [])
+            if 'https://metadata.datadrivendiscovery.org/types/CategoricalData' in semantic_types:
+                LE = preprocessing.LabelEncoder()
+                LE = LE.fit(inputs.iloc[:,column_index])
+                self._training_inputs[:,column_index] = LE.transform(inputs.iloc[:,column_index])  
+            else:
+                temp = list(inputs.iloc[:, column_index].as_matrix())
+                for i in np.arange(len(temp)):
+                    if bool(temp[i]):
+                        self._training_inputs[i,column_index] = float(temp[i])
+                    else:
+                        self._training_inputs[i,column_index] = 'nan'
+            
         self._fitted = False
 
 
@@ -107,7 +135,7 @@ class IPCMBplus_Selector(SupervisedLearnerPrimitiveBase[Inputs, Outputs, Params,
 
     def produce(self, *, inputs: Inputs, timeout: float = None, iterations: int = None) -> Outputs:  # inputs: m x n numpy array
         if self._fitted:
-            return CallResult(inputs[:, self._index])
+            return CallResult(inputs.iloc[:, self._index])
         else:
             raise ValueError('Model should be fitted first.')
 
@@ -140,15 +168,12 @@ class JMIplus_Selector(SupervisedLearnerPrimitiveBase[Inputs, Outputs, Params, H
             {
                 'type': metadata_base.PrimitiveInstallationType.PIP,
                 'package': 'rpi_featureSelection_python_tools',
-	            'version': '2.2.1'
+	            'version': '2.2.2'
             }
         ],
         'python_path': 'd3m.primitives.rpi_featureSelection_python_tools.JMIplus_Selector',
         'algorithm_types': ['MINIMUM_REDUNDANCY_FEATURE_SELECTION'],
         'primitive_family': 'FEATURE_SELECTION',
-        'preconditions': [
-            "NO_CATEGORICAL_VALUES"
-        ]
     })
 
 
@@ -160,8 +185,31 @@ class JMIplus_Selector(SupervisedLearnerPrimitiveBase[Inputs, Outputs, Params, H
         self._fitted = False
 
     def set_training_data(self, *, inputs: Inputs, outputs: Outputs) -> None:
-        self._training_inputs = inputs
-        self._training_outputs = outputs
+        # convert categorical values to numerical values for training labels
+        LE = preprocessing.LabelEncoder()
+        LE = LE.fit(outputs.iloc[:,0])
+        self._training_outputs = LE.transform(outputs.iloc[:,0])
+        
+        # convert cateforical values to numerical values in training data
+        metadata = inputs.metadata
+        [m,n] = inputs.shape
+        self._training_inputs = np.zeros((m,n))
+        for column_index in metadata.get_elements((metadata_base.ALL_ELEMENTS,)):
+            if column_index is metadata_base.ALL_ELEMENTS: 
+                continue
+            column_metadata = metadata.query((metadata_base.ALL_ELEMENTS, column_index))
+            semantic_types = column_metadata.get('semantic_types', [])
+            if 'https://metadata.datadrivendiscovery.org/types/CategoricalData' in semantic_types:
+                LE = preprocessing.LabelEncoder()
+                LE = LE.fit(inputs.iloc[:,column_index])
+                self._training_inputs[:,column_index] = LE.transform(inputs.iloc[:,column_index])  
+            else:
+                temp = list(inputs.iloc[:, column_index].as_matrix())
+                for i in np.arange(len(temp)):
+                    if bool(temp[i]):
+                        self._training_inputs[i,column_index] = float(temp[i])
+                    else:
+                        self._training_inputs[i,column_index] = 'nan'
         self._fitted = False
 
 
@@ -189,7 +237,7 @@ class JMIplus_Selector(SupervisedLearnerPrimitiveBase[Inputs, Outputs, Params, H
 
     def produce(self, *, inputs: Inputs, timeout: float = None, iterations: int = None) -> Outputs:  # inputs: m x n numpy array
         if self._fitted:
-            return CallResult(inputs[:, self._index])
+            return CallResult(inputs.iloc[:, self._index])
         else:
             raise ValueError('Model should be fitted first.')
 
@@ -222,15 +270,12 @@ class STMBplus_Selector(SupervisedLearnerPrimitiveBase[Inputs, Outputs, Params,
             {
                 'type': metadata_base.PrimitiveInstallationType.PIP,
                 'package': 'rpi_featureSelection_python_tools',
-	            'version': '2.2.1'
+	            'version': '2.2.2'
             }
         ],
         'python_path': 'd3m.primitives.rpi_featureSelection_python_tools.STMBplus_Selector',
         'algorithm_types': ['MINIMUM_REDUNDANCY_FEATURE_SELECTION'],
         'primitive_family': 'FEATURE_SELECTION',
-        'preconditions': [
-            "NO_CATEGORICAL_VALUES"
-        ]
     })
 
 
@@ -242,8 +287,31 @@ class STMBplus_Selector(SupervisedLearnerPrimitiveBase[Inputs, Outputs, Params,
         self._fitted = False
 
     def set_training_data(self, *, inputs: Inputs, outputs: Outputs) -> None:
-        self._training_inputs = inputs
-        self._training_outputs = outputs
+        # convert categorical values to numerical values for training labels
+        LE = preprocessing.LabelEncoder()
+        LE = LE.fit(outputs.iloc[:,0])
+        self._training_outputs = LE.transform(outputs.iloc[:,0])
+        
+        # convert cateforical values to numerical values in training data
+        metadata = inputs.metadata
+        [m,n] = inputs.shape
+        self._training_inputs = np.zeros((m,n))
+        for column_index in metadata.get_elements((metadata_base.ALL_ELEMENTS,)):
+            if column_index is metadata_base.ALL_ELEMENTS: 
+                continue
+            column_metadata = metadata.query((metadata_base.ALL_ELEMENTS, column_index))
+            semantic_types = column_metadata.get('semantic_types', [])
+            if 'https://metadata.datadrivendiscovery.org/types/CategoricalData' in semantic_types:
+                LE = preprocessing.LabelEncoder()
+                LE = LE.fit(inputs.iloc[:,column_index])
+                self._training_inputs[:,column_index] = LE.transform(inputs.iloc[:,column_index])  
+            else:
+                temp = list(inputs.iloc[:, column_index].as_matrix())
+                for i in np.arange(len(temp)):
+                    if bool(temp[i]):
+                        self._training_inputs[i,column_index] = float(temp[i])
+                    else:
+                        self._training_inputs[i,column_index] = 'nan'
         self._fitted = False
 
 
@@ -273,7 +341,7 @@ class STMBplus_Selector(SupervisedLearnerPrimitiveBase[Inputs, Outputs, Params,
 
     def produce(self, *, inputs: Inputs, timeout: float = None, iterations: int = None) -> Outputs:  # inputs: m x n numpy array
         if self._fitted:
-            return CallResult(inputs[:, self._index])
+            return CallResult(inputs.iloc[:, self._index])
         else:
             raise ValueError('Model should be fitted first.')
 
@@ -306,15 +374,12 @@ class aSTMBplus_Selector(SupervisedLearnerPrimitiveBase[Inputs, Outputs, Params,
             {
                 'type': metadata_base.PrimitiveInstallationType.PIP,
                 'package': 'rpi_featureSelection_python_tools',
-	            'version': '2.2.1'
+	            'version': '2.2.2'
             }
         ],
         'python_path': 'd3m.primitives.rpi_featureSelection_python_tools.aSTMBplus_Selector',
         'algorithm_types': ['MINIMUM_REDUNDANCY_FEATURE_SELECTION'],
         'primitive_family': 'FEATURE_SELECTION',
-        'preconditions': [
-            "NO_CATEGORICAL_VALUES"
-        ]
     })
 
 
@@ -326,8 +391,31 @@ class aSTMBplus_Selector(SupervisedLearnerPrimitiveBase[Inputs, Outputs, Params,
         self._fitted = False
 
     def set_training_data(self, *, inputs: Inputs, outputs: Outputs) -> None:
-        self._training_inputs = inputs
-        self._training_outputs = outputs
+        # convert categorical values to numerical values for training labels
+        LE = preprocessing.LabelEncoder()
+        LE = LE.fit(outputs.iloc[:,0])
+        self._training_outputs = LE.transform(outputs.iloc[:,0])
+        
+        # convert cateforical values to numerical values in training data
+        metadata = inputs.metadata
+        [m,n] = inputs.shape
+        self._training_inputs = np.zeros((m,n))
+        for column_index in metadata.get_elements((metadata_base.ALL_ELEMENTS,)):
+            if column_index is metadata_base.ALL_ELEMENTS: 
+                continue
+            column_metadata = metadata.query((metadata_base.ALL_ELEMENTS, column_index))
+            semantic_types = column_metadata.get('semantic_types', [])
+            if 'https://metadata.datadrivendiscovery.org/types/CategoricalData' in semantic_types:
+                LE = preprocessing.LabelEncoder()
+                LE = LE.fit(inputs.iloc[:,column_index])
+                self._training_inputs[:,column_index] = LE.transform(inputs.iloc[:,column_index])  
+            else:
+                temp = list(inputs.iloc[:, column_index].as_matrix())
+                for i in np.arange(len(temp)):
+                    if bool(temp[i]):
+                        self._training_inputs[i,column_index] = float(temp[i])
+                    else:
+                        self._training_inputs[i,column_index] = 'nan'
         self._fitted = False
 
 
@@ -355,7 +443,8 @@ class aSTMBplus_Selector(SupervisedLearnerPrimitiveBase[Inputs, Outputs, Params,
 
     def produce(self, *, inputs: Inputs, timeout: float = None, iterations: int = None) -> Outputs:  # inputs: m x n numpy array
         if self._fitted:
-            return CallResult(inputs[:, self._index])
+            #return CallResult(inputs[:, self._index])
+            return CallResult(inputs.iloc[:, self._index])
         else:
             raise ValueError('Model should be fitted first.')
 
@@ -388,15 +477,12 @@ class sSTMBplus_Selector(SupervisedLearnerPrimitiveBase[Inputs, Outputs, Params,
             {
                 'type': metadata_base.PrimitiveInstallationType.PIP,
                 'package': 'rpi_featureSelection_python_tools',
-	            'version': '2.2.1'
+	            'version': '2.2.2'
             }
         ],
         'python_path': 'd3m.primitives.rpi_featureSelection_python_tools.sSTMBplus_Selector',
         'algorithm_types': ['MINIMUM_REDUNDANCY_FEATURE_SELECTION'],
         'primitive_family': 'FEATURE_SELECTION',
-        'preconditions': [
-            "NO_CATEGORICAL_VALUES"
-        ]
     })
 
 
@@ -408,8 +494,31 @@ class sSTMBplus_Selector(SupervisedLearnerPrimitiveBase[Inputs, Outputs, Params,
         self._fitted = False
 
     def set_training_data(self, *, inputs: Inputs, outputs: Outputs) -> None:
-        self._training_inputs = inputs
-        self._training_outputs = outputs
+        # convert categorical values to numerical values for training labels
+        LE = preprocessing.LabelEncoder()
+        LE = LE.fit(outputs.iloc[:,0])
+        self._training_outputs = LE.transform(outputs.iloc[:,0])
+        
+        # convert cateforical values to numerical values in training data
+        metadata = inputs.metadata
+        [m,n] = inputs.shape
+        self._training_inputs = np.zeros((m,n))
+        for column_index in metadata.get_elements((metadata_base.ALL_ELEMENTS,)):
+            if column_index is metadata_base.ALL_ELEMENTS: 
+                continue
+            column_metadata = metadata.query((metadata_base.ALL_ELEMENTS, column_index))
+            semantic_types = column_metadata.get('semantic_types', [])
+            if 'https://metadata.datadrivendiscovery.org/types/CategoricalData' in semantic_types:
+                LE = preprocessing.LabelEncoder()
+                LE = LE.fit(inputs.iloc[:,column_index])
+                self._training_inputs[:,column_index] = LE.transform(inputs.iloc[:,column_index])  
+            else:
+                temp = list(inputs.iloc[:, column_index].as_matrix())
+                for i in np.arange(len(temp)):
+                    if bool(temp[i]):
+                        self._training_inputs[i,column_index] = float(temp[i])
+                    else:
+                        self._training_inputs[i,column_index] = 'nan'
         self._fitted = False
 
 
@@ -436,7 +545,7 @@ class sSTMBplus_Selector(SupervisedLearnerPrimitiveBase[Inputs, Outputs, Params,
 
     def produce(self, *, inputs: Inputs, timeout: float = None, iterations: int = None) -> Outputs:  # inputs: m x n numpy array
         if self._fitted:
-            return CallResult(inputs[:, self._index])
+            return CallResult(inputs.iloc[:, self._index])
         else:
             raise ValueError('Model should be fitted first.')
 
@@ -469,15 +578,12 @@ class pSTMB_Selector(SupervisedLearnerPrimitiveBase[Inputs, Outputs, Params, Hyp
             {
                 'type': metadata_base.PrimitiveInstallationType.PIP,
                 'package': 'rpi_featureSelection_python_tools',
-	            'version': '2.2.1'
+	            'version': '2.2.2'
             }
         ],
         'python_path': 'd3m.primitives.rpi_featureSelection_python_tools.pSTMB_Selector',
         'algorithm_types': ['MINIMUM_REDUNDANCY_FEATURE_SELECTION'],
         'primitive_family': 'FEATURE_SELECTION',
-        'preconditions': [
-            "NO_CATEGORICAL_VALUES"
-        ]
     })
 
 
@@ -489,8 +595,31 @@ class pSTMB_Selector(SupervisedLearnerPrimitiveBase[Inputs, Outputs, Params, Hyp
         self._fitted = False
 
     def set_training_data(self, *, inputs: Inputs, outputs: Outputs) -> None:
-        self._training_inputs = inputs
-        self._training_outputs = outputs
+        # convert categorical values to numerical values for training labels
+        LE = preprocessing.LabelEncoder()
+        LE = LE.fit(outputs.iloc[:,0])
+        self._training_outputs = LE.transform(outputs.iloc[:,0])
+        
+        # convert cateforical values to numerical values in training data
+        metadata = inputs.metadata
+        [m,n] = inputs.shape
+        self._training_inputs = np.zeros((m,n))
+        for column_index in metadata.get_elements((metadata_base.ALL_ELEMENTS,)):
+            if column_index is metadata_base.ALL_ELEMENTS: 
+                continue
+            column_metadata = metadata.query((metadata_base.ALL_ELEMENTS, column_index))
+            semantic_types = column_metadata.get('semantic_types', [])
+            if 'https://metadata.datadrivendiscovery.org/types/CategoricalData' in semantic_types:
+                LE = preprocessing.LabelEncoder()
+                LE = LE.fit(inputs.iloc[:,column_index])
+                self._training_inputs[:,column_index] = LE.transform(inputs.iloc[:,column_index])  
+            else:
+                temp = list(inputs.iloc[:, column_index].as_matrix())
+                for i in np.arange(len(temp)):
+                    if bool(temp[i]):
+                        self._training_inputs[i,column_index] = float(temp[i])
+                    else:
+                        self._training_inputs[i,column_index] = 'nan'
         self._fitted = False
 
 
@@ -518,7 +647,7 @@ class pSTMB_Selector(SupervisedLearnerPrimitiveBase[Inputs, Outputs, Params, Hyp
 
     def produce(self, *, inputs: Inputs, timeout: float = None, iterations: int = None) -> Outputs:  # inputs: m x n numpy array
         if self._fitted:
-            return CallResult(inputs[:, self._index])
+            return CallResult(inputs.iloc[:, self._index])
         else:
             raise ValueError('Model should be fitted first.')
 
@@ -550,15 +679,12 @@ class F_STMB_Selector(SupervisedLearnerPrimitiveBase[Inputs, Outputs, Params, Hy
             {
                 'type': metadata_base.PrimitiveInstallationType.PIP,
                 'package': 'rpi_featureSelection_python_tools',
-	            'version': '2.2.1'
+	            'version': '2.2.2'
             }
         ],
         'python_path': 'd3m.primitives.rpi_featureSelection_python_tools.F_STMB_Selector',
         'algorithm_types': ['MINIMUM_REDUNDANCY_FEATURE_SELECTION'],
         'primitive_family': 'FEATURE_SELECTION',
-        'preconditions': [
-            "NO_CATEGORICAL_VALUES"
-        ]
     })
 
 
@@ -570,8 +696,31 @@ class F_STMB_Selector(SupervisedLearnerPrimitiveBase[Inputs, Outputs, Params, Hy
         self._fitted = False
 
     def set_training_data(self, *, inputs: Inputs, outputs: Outputs) -> None:
-        self._training_inputs = inputs
-        self._training_outputs = outputs
+        # convert categorical values to numerical values for training labels
+        LE = preprocessing.LabelEncoder()
+        LE = LE.fit(outputs.iloc[:,0])
+        self._training_outputs = LE.transform(outputs.iloc[:,0])
+        
+        # convert cateforical values to numerical values in training data
+        metadata = inputs.metadata
+        [m,n] = inputs.shape
+        self._training_inputs = np.zeros((m,n))
+        for column_index in metadata.get_elements((metadata_base.ALL_ELEMENTS,)):
+            if column_index is metadata_base.ALL_ELEMENTS: 
+                continue
+            column_metadata = metadata.query((metadata_base.ALL_ELEMENTS, column_index))
+            semantic_types = column_metadata.get('semantic_types', [])
+            if 'https://metadata.datadrivendiscovery.org/types/CategoricalData' in semantic_types:
+                LE = preprocessing.LabelEncoder()
+                LE = LE.fit(inputs.iloc[:,column_index])
+                self._training_inputs[:,column_index] = LE.transform(inputs.iloc[:,column_index])  
+            else:
+                temp = list(inputs.iloc[:, column_index].as_matrix())
+                for i in np.arange(len(temp)):
+                    if bool(temp[i]):
+                        self._training_inputs[i,column_index] = float(temp[i])
+                    else:
+                        self._training_inputs[i,column_index] = 'nan'
         self._fitted = False
 
 
@@ -599,7 +748,7 @@ class F_STMB_Selector(SupervisedLearnerPrimitiveBase[Inputs, Outputs, Params, Hy
 
     def produce(self, *, inputs: Inputs, timeout: float = None, iterations: int = None) -> Outputs:  # inputs: m x n numpy array
         if self._fitted:
-            return CallResult(inputs[:, self._index])
+            return CallResult(inputs.iloc[:, self._index])
         else:
             raise ValueError('Model should be fitted first.')
 
@@ -633,15 +782,12 @@ class F_aSTMB_Selector(SupervisedLearnerPrimitiveBase[Inputs, Outputs, Params, H
             {
                 'type': metadata_base.PrimitiveInstallationType.PIP,
                 'package': 'rpi_featureSelection_python_tools',
-	            'version': '2.2.1'
+	            'version': '2.2.2'
             }
         ],
         'python_path': 'd3m.primitives.rpi_featureSelection_python_tools.F_aSTMB_Selector',
         'algorithm_types': ['MINIMUM_REDUNDANCY_FEATURE_SELECTION'],
         'primitive_family': 'FEATURE_SELECTION',
-        'preconditions': [
-            "NO_CATEGORICAL_VALUES"
-        ]
     })
 
 
@@ -653,8 +799,31 @@ class F_aSTMB_Selector(SupervisedLearnerPrimitiveBase[Inputs, Outputs, Params, H
         self._fitted = False
 
     def set_training_data(self, *, inputs: Inputs, outputs: Outputs) -> None:
-        self._training_inputs = inputs
-        self._training_outputs = outputs
+        # convert categorical values to numerical values for training labels
+        LE = preprocessing.LabelEncoder()
+        LE = LE.fit(outputs.iloc[:,0])
+        self._training_outputs = LE.transform(outputs.iloc[:,0])
+        
+        # convert cateforical values to numerical values in training data
+        metadata = inputs.metadata
+        [m,n] = inputs.shape
+        self._training_inputs = np.zeros((m,n))
+        for column_index in metadata.get_elements((metadata_base.ALL_ELEMENTS,)):
+            if column_index is metadata_base.ALL_ELEMENTS: 
+                continue
+            column_metadata = metadata.query((metadata_base.ALL_ELEMENTS, column_index))
+            semantic_types = column_metadata.get('semantic_types', [])
+            if 'https://metadata.datadrivendiscovery.org/types/CategoricalData' in semantic_types:
+                LE = preprocessing.LabelEncoder()
+                LE = LE.fit(inputs.iloc[:,column_index])
+                self._training_inputs[:,column_index] = LE.transform(inputs.iloc[:,column_index])  
+            else:
+                temp = list(inputs.iloc[:, column_index].as_matrix())
+                for i in np.arange(len(temp)):
+                    if bool(temp[i]):
+                        self._training_inputs[i,column_index] = float(temp[i])
+                    else:
+                        self._training_inputs[i,column_index] = 'nan'
         self._fitted = False
 
 
@@ -682,7 +851,7 @@ class F_aSTMB_Selector(SupervisedLearnerPrimitiveBase[Inputs, Outputs, Params, H
 
     def produce(self, *, inputs: Inputs, timeout: float = None, iterations: int = None) -> Outputs:  # inputs: m x n numpy array
         if self._fitted:
-            return CallResult(inputs[:, self._index])
+            return CallResult(inputs.iloc[:, self._index])
         else:
             raise ValueError('Model should be fitted first.')
 
@@ -715,15 +884,12 @@ class F_sSTMB_Selector(SupervisedLearnerPrimitiveBase[Inputs, Outputs, Params, H
             {
                 'type': metadata_base.PrimitiveInstallationType.PIP,
                 'package': 'rpi_featureSelection_python_tools',
-	            'version': '2.2.1'
+	            'version': '2.2.2'
             }
         ],
         'python_path': 'd3m.primitives.rpi_featureSelection_python_tools.F_sSTMB_Selector',
         'algorithm_types': ['MINIMUM_REDUNDANCY_FEATURE_SELECTION'],
         'primitive_family': 'FEATURE_SELECTION',
-        'preconditions': [
-            "NO_CATEGORICAL_VALUES"
-        ]
     })
 
 
@@ -735,8 +901,31 @@ class F_sSTMB_Selector(SupervisedLearnerPrimitiveBase[Inputs, Outputs, Params, H
         self._fitted = False
 
     def set_training_data(self, *, inputs: Inputs, outputs: Outputs) -> None:
-        self._training_inputs = inputs
-        self._training_outputs = outputs
+       # convert categorical values to numerical values for training labels
+        LE = preprocessing.LabelEncoder()
+        LE = LE.fit(outputs.iloc[:,0])
+        self._training_outputs = LE.transform(outputs.iloc[:,0])
+        
+        # convert cateforical values to numerical values in training data
+        metadata = inputs.metadata
+        [m,n] = inputs.shape
+        self._training_inputs = np.zeros((m,n))
+        for column_index in metadata.get_elements((metadata_base.ALL_ELEMENTS,)):
+            if column_index is metadata_base.ALL_ELEMENTS: 
+                continue
+            column_metadata = metadata.query((metadata_base.ALL_ELEMENTS, column_index))
+            semantic_types = column_metadata.get('semantic_types', [])
+            if 'https://metadata.datadrivendiscovery.org/types/CategoricalData' in semantic_types:
+                LE = preprocessing.LabelEncoder()
+                LE = LE.fit(inputs.iloc[:,column_index])
+                self._training_inputs[:,column_index] = LE.transform(inputs.iloc[:,column_index])  
+            else:
+                temp = list(inputs.iloc[:, column_index].as_matrix())
+                for i in np.arange(len(temp)):
+                    if bool(temp[i]):
+                        self._training_inputs[i,column_index] = float(temp[i])
+                    else:
+                        self._training_inputs[i,column_index] = 'nan'
         self._fitted = False
 
 
@@ -764,7 +953,7 @@ class F_sSTMB_Selector(SupervisedLearnerPrimitiveBase[Inputs, Outputs, Params, H
 
     def produce(self, *, inputs: Inputs, timeout: float = None, iterations: int = None) -> Outputs:  # inputs: m x n numpy array
         if self._fitted:
-            return CallResult(inputs[:, self._index])
+            return CallResult(inputs.iloc[:, self._index])
         else:
             raise ValueError('Model should be fitted first.')
 
@@ -797,15 +986,12 @@ class JMIp_Selector(SupervisedLearnerPrimitiveBase[Inputs, Outputs, Params, Hype
             {
                 'type': metadata_base.PrimitiveInstallationType.PIP,
                 'package': 'rpi_featureSelection_python_tools',
-	            'version': '2.2.1'
+	            'version': '2.2.2'
             }
         ],
         'python_path': 'd3m.primitives.rpi_featureSelection_python_tools.JMIp_Selector',
         'algorithm_types': ['MINIMUM_REDUNDANCY_FEATURE_SELECTION'],
         'primitive_family': 'FEATURE_SELECTION',
-        'preconditions': [
-            "NO_CATEGORICAL_VALUES"
-        ]
     })
 
 
@@ -817,8 +1003,31 @@ class JMIp_Selector(SupervisedLearnerPrimitiveBase[Inputs, Outputs, Params, Hype
         self._fitted = False
 
     def set_training_data(self, *, inputs: Inputs, outputs: Outputs) -> None:
-        self._training_inputs = inputs
-        self._training_outputs = outputs
+        # convert categorical values to numerical values for training labels
+        LE = preprocessing.LabelEncoder()
+        LE = LE.fit(outputs.iloc[:,0])
+        self._training_outputs = LE.transform(outputs.iloc[:,0])
+        
+        # convert cateforical values to numerical values in training data
+        metadata = inputs.metadata
+        [m,n] = inputs.shape
+        self._training_inputs = np.zeros((m,n))
+        for column_index in metadata.get_elements((metadata_base.ALL_ELEMENTS,)):
+            if column_index is metadata_base.ALL_ELEMENTS: 
+                continue
+            column_metadata = metadata.query((metadata_base.ALL_ELEMENTS, column_index))
+            semantic_types = column_metadata.get('semantic_types', [])
+            if 'https://metadata.datadrivendiscovery.org/types/CategoricalData' in semantic_types:
+                LE = preprocessing.LabelEncoder()
+                LE = LE.fit(inputs.iloc[:,column_index])
+                self._training_inputs[:,column_index] = LE.transform(inputs.iloc[:,column_index])  
+            else:
+                temp = list(inputs.iloc[:, column_index].as_matrix())
+                for i in np.arange(len(temp)):
+                    if bool(temp[i]):
+                        self._training_inputs[i,column_index] = float(temp[i])
+                    else:
+                        self._training_inputs[i,column_index] = 'nan'
         self._fitted = False
 
 
@@ -846,7 +1055,7 @@ class JMIp_Selector(SupervisedLearnerPrimitiveBase[Inputs, Outputs, Params, Hype
 
     def produce(self, *, inputs: Inputs, timeout: float = None, iterations: int = None) -> Outputs:  # inputs: m x n numpy array
         if self._fitted:
-            return CallResult(inputs[:, self._index])
+            return CallResult(inputs.iloc[:, self._index])
         else:
             raise ValueError('Model should be fitted first.')
 
